local E, L, V, P, G = unpack(ElvUI)
local TT = E:GetModule('Tooltip')

local plugin = E:NewModule("Tooltip2Bag", "AceTimer-3.0", "AceHook-3.0")
local AddOnName = (...)
local DEBUG = false

local _G = _G
local OpenAllBags = OpenAllBags
local CloseAllBags = CloseAllBags
local GetNumAddOns = GetNumAddOns
local IsAddOnLoaded = IsAddOnLoaded
local GetAddOnInfo = GetAddOnInfo
local GameTooltip = GameTooltip
local ElvUI_ContainerFrame = ElvUI_ContainerFrame
local print, pairs, type = print, pairs, type


local bagAddonsSupported = {
	["AdiBags"] = "AdiBagsContainer1",
	["ArkInventory"] = "ARKINV_Frame1",
	["Baggins"] = "BagginsBag1",
	["Bagnon"] = "BagnonFrameinventory",
	
	["BaudBag"] = {
		["frameName"] = "BaudBagContainer1_1",
		["showHook"] = function()
			_G.ElvUI_ContainerFrame = _G.BaudBagContainer1_1Backdrop
		end,
		["hideHook"] = function()
			_G.ElvUI_ContainerFrame = nil
		end
	},
	
	["Combuctor"] = "CombuctorFrameinventory",
	["djbags"] = "DJBagsBagContainer",
	["Inventorian"] = "InventorianBagFrame",
	["LiteBag"] = "LiteBagInventory",
	
	["OneBag3"] = "OneBagFrame",
}

-- Default hook when showing/hiding a bag frame
local function noop() end

-- Default post-show function
local function afterShowHook()
	TT:GameTooltip_SetDefaultAnchor(GameTooltip)
end

-- Default post-hide function
local function afterHideHook()
	GameTooltip:ClearAllPoints()
	TT:GameTooltip_SetDefaultAnchor(GameTooltip)
end


--[[
	Main function to check for supported bag addons and expose them to 
	ElvUI. We do as little special processing as possible and so the 
	entire purpose of this is to get the correct bag addon frame assigned 
	to ElvUI_ContainerFrame.
]]
function plugin:checkAddons()
	self.timerCount = self.timerCount + 1
	if DEBUG then print("PULSE: "..self.timerCount) end
	
	if self.timerCount > self.maxTimerCount then
		-- Didn't find anything we support
		self:CancelTimer(self.timer)
	elseif self.timerCount >= self.maxTimerCount - 1 then
		-- Some bag addons don't create their frame until needed
		OpenAllBags()
		CloseAllBags()
	end

	-- See if any addons we support are loaded
	for i = 1, GetNumAddOns() do
	   if IsAddOnLoaded(i) then
		  local name = (GetAddOnInfo(i))

		  for k, v in pairs(bagAddonsSupported) do
			if k == name then
				if DEBUG then print("NAME FOUND: "..name) end
				
				local frame = nil
				local showHook = noop
				local hideHook = noop
				local frameName = v
				
				if type(v) == "table" then
					frameName = v["frameName"]
					showHook = v["showHook"]
					hideHook = v["hideHook"]
				end
				
				frame = _G[frameName]

				if frame ~= nil then
					if DEBUG then print("MATCH FOUND: "..name) end

					self:SecureHook(frame, "Show", function()
						showHook()
						afterShowHook()
					end)
					
					self:SecureHook(frame, "Hide", function()
						hideHook()
						afterHideHook()
					end)
					
					_G.ElvUI_ContainerFrame = frame
					afterHideHook()
					self:CancelTimer(self.timer)
					return
				end
			end
		  end
	   end
	end
end

function plugin:Initialize()
	if ElvUI_ContainerFrame then return end
	LibStub("LibElvUIPlugin-1.0"):RegisterPlugin(AddOnName)

	--[[
		There are no good ways to hook into the various bag addons so we
		just run a short timer to scan for supported addons.
	--]]
	self.timerPulse = .5
	self.timerCount = 0
	self.maxTimerCount = 40
	self.timer = self:ScheduleRepeatingTimer("checkAddons", self.timerPulse)
end

E:RegisterModule(plugin:GetName()) 
